@@ -39,8 +39,6 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: error_message.c,v 1.3 2010/01/06 17:01:28 murch Exp $
- *
  * Copyright 1987 by the Student Information Processing Board
  * of the Massachusetts Institute of Technology
  *
@@ -64,7 +62,7 @@ static char buffer[25];
 
 extern struct et_list * _et_list;
 
-const char * INTERFACE error_message (code)
+EXPORTED const char * INTERFACE error_message (code)
 long code;
 {
     int offset;
