@@ -39,8 +39,6 @@
  * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * $Id: cyr_synclog.c,v 1.6 2009/12/16 02:51:39 brong Exp $
- *
  * Originally written by Bron Gondwana <brong@fastmail.fm>
  */
 
@@ -52,25 +50,23 @@
 #include <unistd.h>
 #endif
 
+#include "exitcodes.h"
 #include "global.h"
 #include "sync_log.h"
 #include "util.h"
 #include "xmalloc.h"
 
-/* config.c stuff */
-const int config_need_data = 0;
-
 int main(int argc, char *argv[])
 {
     char *alt_config = NULL;
     char cmd = '\0';
     char opt;
 
-    if ((geteuid()) == 0 && (become_cyrus() != 0)) {
+    if ((geteuid()) == 0 && (become_cyrus(/*is_master*/0) != 0)) {
 	fatal("must run as the Cyrus user", EC_USAGE);
     }
 
-    while ((opt = getopt(argc, argv, "C:uvmacqnsb")) != EOF) {
+    while ((opt = getopt(argc, argv, "C:uUvmMacqnsb")) != EOF) {
 	switch (opt) {
 	case 'C': /* alt config file */
 	    alt_config = optarg;
@@ -78,12 +74,18 @@ int main(int argc, char *argv[])
 	case 'u': /* User */
 	    cmd = 'u';
 	    break;
+	case 'U': /* UnUser */
+	    cmd = 'U';
+	    break;
 	case 'v': /* sieVe */
 	    cmd = 'v';
 	    break;
 	case 'm': /* Mailbox */
 	    cmd = 'm';
 	    break;
+	case 'M': /* UnMailbox */
+	    cmd = 'M';
+	    break;
 	case 'a': /* Append */
 	    cmd = 'a';
 	    break;
@@ -112,10 +114,10 @@ int main(int argc, char *argv[])
 	fprintf(stderr, "\n");
 	fprintf(stderr, "types:\n");
 	fprintf(stderr, "  -u   USER\n");
+	fprintf(stderr, "  -U   UNUSER\n");
 	fprintf(stderr, "  -v   SIEVE\n");
 	fprintf(stderr, "  -m   MAILBOX\n");
-	fprintf(stderr, "  -a   APPEND\n");
-	fprintf(stderr, "  -c   ACL\n");
+	fprintf(stderr, "  -M   UNMAILBOX\n");
 	fprintf(stderr, "  -q   QUOTA\n");
 	fprintf(stderr, "  -n   ANNOTATION\n");
 	fprintf(stderr, "  -s   SEEN\n");
@@ -126,24 +128,24 @@ int main(int argc, char *argv[])
 	exit(-1);
     }
 
-    cyrus_init(alt_config, "cyr_synclog", 0);
+    cyrus_init(alt_config, "cyr_synclog", 0, 0);
     sync_log_init();
 
     switch(cmd) {
 	case 'u': /* User */
 	    sync_log_user(argv[optind]);
 	    break;
+	case 'U': /* UnUser */
+	    sync_log_unuser(argv[optind]);
+	    break;
 	case 'v': /* sieVe */
 	    sync_log_sieve(argv[optind]);
 	    break;
 	case 'm': /* Mailbox */
 	    sync_log_mailbox(argv[optind]);
 	    break;
-	case 'a': /* Append */
-	    sync_log_append(argv[optind]);
-	    break;
-	case 'c': /* aCl */
-	    sync_log_acl(argv[optind]);
+	case 'M': /* UnMailbox */
+	    sync_log_unmailbox(argv[optind]);
 	    break;
 	case 'q': /* Quota */
 	    sync_log_quota(argv[optind]);
