@@ -115,7 +115,7 @@ uint32_t mboxlist_string_to_mbtype(const char *string);
 /* Lookup 'name' in the mailbox list. */
 int mboxlist_lookup(const char *name, mbentry_t **mbentryptr,
 		    struct txn **tid);
-int mboxlist_lookup_allow_reserved(const char *name,
+int mboxlist_lookup_allow_all(const char *name,
 				   mbentry_t **mbentryptr,
 				   struct txn **tid);
 
@@ -165,10 +165,11 @@ int mboxlist_createsync(const char *name, int mbtype,
 /* prepare MailboxDelete notification if mboxevent is not NULL */
 int
 mboxlist_delayed_deletemailbox(const char *name, int isadmin, const char *userid, 
-                               struct auth_state *auth_state,
-                               struct mboxevent *mboxevent,
+			       struct auth_state *auth_state,
+			       struct mboxevent *mboxevent,
 			       int checkacl,
-                               int force);
+			       int localonly,
+			       int force);
 /* Delete a mailbox. */
 /* setting local_only disables any communication with the mupdate server
  * and deletes the mailbox from the filesystem regardless of if it is
@@ -189,6 +190,7 @@ int mboxlist_renamemailbox(const char *oldname, const char *newname,
 			   int isadmin, const char *userid,
 			   struct auth_state *auth_state,
 			   struct mboxevent *mboxevent,
+			   int local_only,
 			   int forceuser, int ignorequota);
 
 /* change ACL */
